@@ -6,7 +6,7 @@ import (
 	"time"
 )
 
-var m, _ = New(
+var m = New(
 	"mymetric",
 	map[string]string{
 		"host":        "host.example.com",
