@@ -1,22 +1,21 @@
-//go:generate go run -tags generate snmp_mocks_generate.go
 package snmp
 
 import (
 	"fmt"
 	"net"
-	"os/exec"
+	"path/filepath"
 	"sync"
 	"testing"
 	"time"
 
 	"github.com/gosnmp/gosnmp"
+	"github.com/influxdata/toml"
+	"github.com/stretchr/testify/require"
+
 	"github.com/influxdata/telegraf/config"
 	"github.com/influxdata/telegraf/internal/snmp"
 	"github.com/influxdata/telegraf/plugins/inputs"
 	"github.com/influxdata/telegraf/testutil"
-	"github.com/influxdata/toml"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 type testSNMPConnection struct {
@@ -63,24 +62,42 @@ func (tsc *testSNMPConnection) Walk(oid string, wf gosnmp.WalkFunc) error {
 var tsc = &testSNMPConnection{
 	host: "tsc",
 	values: map[string]interface{}{
-		".1.0.0.0.1.1.0":     "foo",
-		".1.0.0.0.1.1.1":     []byte("bar"),
-		".1.0.0.0.1.1.2":     []byte(""),
-		".1.0.0.0.1.102":     "bad",
-		".1.0.0.0.1.2.0":     1,
-		".1.0.0.0.1.2.1":     2,
-		".1.0.0.0.1.2.2":     0,
-		".1.0.0.0.1.3.0":     "0.123",
-		".1.0.0.0.1.3.1":     "0.456",
-		".1.0.0.0.1.3.2":     "0.000",
-		".1.0.0.0.1.3.3":     "9.999",
-		".1.0.0.0.1.5.0":     123456,
-		".1.0.0.1.1":         "baz",
-		".1.0.0.1.2":         234,
-		".1.0.0.1.3":         []byte("byte slice"),
-		".1.0.0.2.1.5.0.9.9": 11,
-		".1.0.0.2.1.5.1.9.9": 22,
-		".1.0.0.0.1.6.0":     ".1.0.0.0.1.7",
+		".1.3.6.1.2.1.3.1.1.1.0": "foo",
+		".1.3.6.1.2.1.3.1.1.1.1": []byte("bar"),
+		".1.3.6.1.2.1.3.1.1.1.2": []byte(""),
+		".1.3.6.1.2.1.3.1.1.102": "bad",
+		".1.3.6.1.2.1.3.1.1.2.0": 1,
+		".1.3.6.1.2.1.3.1.1.2.1": 2,
+		".1.3.6.1.2.1.3.1.1.2.2": 0,
+		".1.3.6.1.2.1.3.1.1.3.0": "1.3.6.1.2.1.3.1.1.3",
+		".1.3.6.1.2.1.3.1.1.5.0": 123456,
+		".1.0.0.0.1.1.0":         "foo",
+		".1.0.0.0.1.1.1":         []byte("bar"),
+		".1.0.0.0.1.1.2":         []byte(""),
+		".1.0.0.0.1.102":         "bad",
+		".1.0.0.0.1.2.0":         1,
+		".1.0.0.0.1.2.1":         2,
+		".1.0.0.0.1.2.2":         0,
+		".1.0.0.0.1.3.0":         "0.123",
+		".1.0.0.0.1.3.1":         "0.456",
+		".1.0.0.0.1.3.2":         "0.000",
+		".1.0.0.0.1.3.3":         "9.999",
+		".1.0.0.0.1.5.0":         123456,
+		".1.0.0.1.1":             "baz",
+		".1.0.0.1.2":             234,
+		".1.0.0.1.3":             []byte("byte slice"),
+		".1.0.0.2.1.5.0.9.9":     11,
+		".1.0.0.2.1.5.1.9.9":     22,
+		".1.0.0.0.1.6.0":         ".1.0.0.0.1.7",
+		".1.0.0.3.1.1.10":        "instance",
+		".1.0.0.3.1.1.11":        "instance2",
+		".1.0.0.3.1.1.12":        "instance3",
+		".1.0.0.3.1.2.10":        10,
+		".1.0.0.3.1.2.11":        20,
+		".1.0.0.3.1.2.12":        20,
+		".1.0.0.3.1.3.10":        1,
+		".1.0.0.3.1.3.11":        2,
+		".1.0.0.3.1.3.12":        3,
 	},
 }
 
@@ -95,6 +112,7 @@ func TestSampleConfig(t *testing.T) {
 		ClientConfig: snmp.ClientConfig{
 			Timeout:        config.Duration(5 * time.Second),
 			Version:        2,
+			Path:           []string{"/usr/share/snmp/mibs"},
 			Community:      "public",
 			MaxRepetitions: 10,
 			Retries:        3,
@@ -105,6 +123,18 @@ func TestSampleConfig(t *testing.T) {
 }
 
 func TestFieldInit(t *testing.T) {
+	testDataPath, err := filepath.Abs("./testdata")
+	require.NoError(t, err)
+	s := &Snmp{
+		ClientConfig: snmp.ClientConfig{
+			Path: []string{testDataPath},
+		},
+		Log: &testutil.Logger{},
+	}
+
+	err = s.Init()
+	require.NoError(t, err)
+
 	translations := []struct {
 		inputOid           string
 		inputName          string
@@ -116,124 +146,130 @@ func TestFieldInit(t *testing.T) {
 		{".1.2.3", "foo", "", ".1.2.3", "foo", ""},
 		{".iso.2.3", "foo", "", ".1.2.3", "foo", ""},
 		{".1.0.0.0.1.1", "", "", ".1.0.0.0.1.1", "server", ""},
-		{".1.0.0.0.1.1.0", "", "", ".1.0.0.0.1.1.0", "server.0", ""},
-		{".999", "", "", ".999", ".999", ""},
-		{"TEST::server", "", "", ".1.0.0.0.1.1", "server", ""},
-		{"TEST::server.0", "", "", ".1.0.0.0.1.1.0", "server.0", ""},
-		{"TEST::server", "foo", "", ".1.0.0.0.1.1", "foo", ""},
 		{"IF-MIB::ifPhysAddress.1", "", "", ".1.3.6.1.2.1.2.2.1.6.1", "ifPhysAddress.1", "hwaddr"},
 		{"IF-MIB::ifPhysAddress.1", "", "none", ".1.3.6.1.2.1.2.2.1.6.1", "ifPhysAddress.1", "none"},
 		{"BRIDGE-MIB::dot1dTpFdbAddress.1", "", "", ".1.3.6.1.2.1.17.4.3.1.1.1", "dot1dTpFdbAddress.1", "hwaddr"},
 		{"TCP-MIB::tcpConnectionLocalAddress.1", "", "", ".1.3.6.1.2.1.6.19.1.2.1", "tcpConnectionLocalAddress.1", "ipaddr"},
+		{".999", "", "", ".999", ".999", ""},
 	}
 
 	for _, txl := range translations {
 		f := Field{Oid: txl.inputOid, Name: txl.inputName, Conversion: txl.inputConversion}
 		err := f.init()
-		if !assert.NoError(t, err, "inputOid='%s' inputName='%s'", txl.inputOid, txl.inputName) {
-			continue
-		}
-		assert.Equal(t, txl.expectedOid, f.Oid, "inputOid='%s' inputName='%s' inputConversion='%s'", txl.inputOid, txl.inputName, txl.inputConversion)
-		assert.Equal(t, txl.expectedName, f.Name, "inputOid='%s' inputName='%s' inputConversion='%s'", txl.inputOid, txl.inputName, txl.inputConversion)
+		require.NoError(t, err, "inputOid='%s' inputName='%s'", txl.inputOid, txl.inputName)
+
+		require.Equal(t, txl.expectedOid, f.Oid, "inputOid='%s' inputName='%s' inputConversion='%s'", txl.inputOid, txl.inputName, txl.inputConversion)
+		require.Equal(t, txl.expectedName, f.Name, "inputOid='%s' inputName='%s' inputConversion='%s'", txl.inputOid, txl.inputName, txl.inputConversion)
 	}
 }
 
 func TestTableInit(t *testing.T) {
-	tbl := Table{
-		Oid: ".1.0.0.0",
-		Fields: []Field{
-			{Oid: ".999", Name: "foo"},
-			{Oid: "TEST::description", Name: "description", IsTag: true},
-		},
-	}
-	err := tbl.Init()
+	testDataPath, err := filepath.Abs("./testdata")
 	require.NoError(t, err)
 
-	assert.Equal(t, "testTable", tbl.Name)
+	s := &Snmp{
+		ClientConfig: snmp.ClientConfig{
+			Path: []string{testDataPath},
+		},
+		Tables: []Table{
+			{Oid: ".1.3.6.1.2.1.3.1",
+				Fields: []Field{
+					{Oid: ".999", Name: "foo"},
+					{Oid: ".1.3.6.1.2.1.3.1.1.1", Name: "atIfIndex", IsTag: true},
+					{Oid: "RFC1213-MIB::atPhysAddress", Name: "atPhysAddress"},
+				}},
+		},
+	}
+	err = s.Init()
+	require.NoError(t, err)
 
-	assert.Len(t, tbl.Fields, 5)
-	assert.Contains(t, tbl.Fields, Field{Oid: ".999", Name: "foo", initialized: true})
-	assert.Contains(t, tbl.Fields, Field{Oid: ".1.0.0.0.1.1", Name: "server", IsTag: true, initialized: true})
-	assert.Contains(t, tbl.Fields, Field{Oid: ".1.0.0.0.1.2", Name: "connections", initialized: true})
-	assert.Contains(t, tbl.Fields, Field{Oid: ".1.0.0.0.1.3", Name: "latency", initialized: true})
-	assert.Contains(t, tbl.Fields, Field{Oid: ".1.0.0.0.1.4", Name: "description", IsTag: true, initialized: true})
+	require.Equal(t, "atTable", s.Tables[0].Name)
+
+	require.Len(t, s.Tables[0].Fields, 5)
+	require.Contains(t, s.Tables[0].Fields, Field{Oid: ".999", Name: "foo", initialized: true})
+	require.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.3.6.1.2.1.3.1.1.1", Name: "atIfIndex", initialized: true, IsTag: true})
+	require.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.3.6.1.2.1.3.1.1.2", Name: "atPhysAddress", initialized: true, Conversion: "hwaddr"})
+	require.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.3.6.1.2.1.3.1.1.3", Name: "atNetAddress", initialized: true, IsTag: true})
 }
 
 func TestSnmpInit(t *testing.T) {
+	testDataPath, err := filepath.Abs("./testdata")
+	require.NoError(t, err)
+
 	s := &Snmp{
 		Tables: []Table{
-			{Oid: "TEST::testTable"},
+			{Oid: "RFC1213-MIB::atTable"},
 		},
 		Fields: []Field{
-			{Oid: "TEST::hostname"},
+			{Oid: "RFC1213-MIB::atPhysAddress"},
+		},
+		ClientConfig: snmp.ClientConfig{
+			Path: []string{testDataPath},
 		},
 	}
 
-	err := s.init()
+	err = s.Init()
 	require.NoError(t, err)
 
-	assert.Len(t, s.Tables[0].Fields, 4)
-	assert.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.0.0.0.1.1", Name: "server", IsTag: true, initialized: true})
-	assert.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.0.0.0.1.2", Name: "connections", initialized: true})
-	assert.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.0.0.0.1.3", Name: "latency", initialized: true})
-	assert.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.0.0.0.1.4", Name: "description", initialized: true})
+	require.Len(t, s.Tables[0].Fields, 3)
+	require.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.3.6.1.2.1.3.1.1.1", Name: "atIfIndex", IsTag: true, initialized: true})
+	require.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.3.6.1.2.1.3.1.1.2", Name: "atPhysAddress", initialized: true, Conversion: "hwaddr"})
+	require.Contains(t, s.Tables[0].Fields, Field{Oid: ".1.3.6.1.2.1.3.1.1.3", Name: "atNetAddress", IsTag: true, initialized: true})
 
-	assert.Equal(t, Field{
-		Oid:         ".1.0.0.1.1",
-		Name:        "hostname",
+	require.Equal(t, Field{
+		Oid:         ".1.3.6.1.2.1.3.1.1.2",
+		Name:        "atPhysAddress",
+		Conversion:  "hwaddr",
 		initialized: true,
 	}, s.Fields[0])
 }
 
 func TestSnmpInit_noTranslate(t *testing.T) {
-	// override execCommand so it returns exec.ErrNotFound
-	defer func(ec func(string, ...string) *exec.Cmd) { execCommand = ec }(execCommand)
-	execCommand = func(_ string, _ ...string) *exec.Cmd {
-		return exec.Command("snmptranslateExecErrNotFound")
-	}
-
 	s := &Snmp{
 		Fields: []Field{
-			{Oid: ".1.1.1.1", Name: "one", IsTag: true},
-			{Oid: ".1.1.1.2", Name: "two"},
-			{Oid: ".1.1.1.3"},
+			{Oid: ".9.1.1.1.1", Name: "one", IsTag: true},
+			{Oid: ".9.1.1.1.2", Name: "two"},
+			{Oid: ".9.1.1.1.3"},
 		},
 		Tables: []Table{
 			{Name: "testing",
 				Fields: []Field{
-					{Oid: ".1.1.1.4", Name: "four", IsTag: true},
-					{Oid: ".1.1.1.5", Name: "five"},
-					{Oid: ".1.1.1.6"},
+					{Oid: ".9.1.1.1.4", Name: "four", IsTag: true},
+					{Oid: ".9.1.1.1.5", Name: "five"},
+					{Oid: ".9.1.1.1.6"},
 				}},
 		},
+		ClientConfig: snmp.ClientConfig{
+			Path: []string{},
+		},
 	}
 
-	err := s.init()
+	err := s.Init()
 	require.NoError(t, err)
 
-	assert.Equal(t, ".1.1.1.1", s.Fields[0].Oid)
-	assert.Equal(t, "one", s.Fields[0].Name)
-	assert.Equal(t, true, s.Fields[0].IsTag)
+	require.Equal(t, ".9.1.1.1.1", s.Fields[0].Oid)
+	require.Equal(t, "one", s.Fields[0].Name)
+	require.Equal(t, true, s.Fields[0].IsTag)
 
-	assert.Equal(t, ".1.1.1.2", s.Fields[1].Oid)
-	assert.Equal(t, "two", s.Fields[1].Name)
-	assert.Equal(t, false, s.Fields[1].IsTag)
+	require.Equal(t, ".9.1.1.1.2", s.Fields[1].Oid)
+	require.Equal(t, "two", s.Fields[1].Name)
+	require.Equal(t, false, s.Fields[1].IsTag)
 
-	assert.Equal(t, ".1.1.1.3", s.Fields[2].Oid)
-	assert.Equal(t, ".1.1.1.3", s.Fields[2].Name)
-	assert.Equal(t, false, s.Fields[2].IsTag)
+	require.Equal(t, ".9.1.1.1.3", s.Fields[2].Oid)
+	require.Equal(t, ".9.1.1.1.3", s.Fields[2].Name)
+	require.Equal(t, false, s.Fields[2].IsTag)
 
-	assert.Equal(t, ".1.1.1.4", s.Tables[0].Fields[0].Oid)
-	assert.Equal(t, "four", s.Tables[0].Fields[0].Name)
-	assert.Equal(t, true, s.Tables[0].Fields[0].IsTag)
+	require.Equal(t, ".9.1.1.1.4", s.Tables[0].Fields[0].Oid)
+	require.Equal(t, "four", s.Tables[0].Fields[0].Name)
+	require.Equal(t, true, s.Tables[0].Fields[0].IsTag)
 
-	assert.Equal(t, ".1.1.1.5", s.Tables[0].Fields[1].Oid)
-	assert.Equal(t, "five", s.Tables[0].Fields[1].Name)
-	assert.Equal(t, false, s.Tables[0].Fields[1].IsTag)
+	require.Equal(t, ".9.1.1.1.5", s.Tables[0].Fields[1].Oid)
+	require.Equal(t, "five", s.Tables[0].Fields[1].Name)
+	require.Equal(t, false, s.Tables[0].Fields[1].IsTag)
 
-	assert.Equal(t, ".1.1.1.6", s.Tables[0].Fields[2].Oid)
-	assert.Equal(t, ".1.1.1.6", s.Tables[0].Fields[2].Name)
-	assert.Equal(t, false, s.Tables[0].Fields[2].IsTag)
+	require.Equal(t, ".9.1.1.1.6", s.Tables[0].Fields[2].Oid)
+	require.Equal(t, ".9.1.1.1.6", s.Tables[0].Fields[2].Name)
+	require.Equal(t, false, s.Tables[0].Fields[2].IsTag)
 }
 
 func TestSnmpInit_noName_noOid(t *testing.T) {
@@ -247,11 +283,15 @@ func TestSnmpInit_noName_noOid(t *testing.T) {
 		},
 	}
 
-	err := s.init()
+	err := s.Init()
 	require.Error(t, err)
 }
 
 func TestGetSNMPConnection_v2(t *testing.T) {
+	if testing.Short() {
+		t.Skip("Skipping integration test in short mode")
+	}
+
 	s := &Snmp{
 		Agents: []string{"1.2.3.4:567", "1.2.3.4", "udp://127.0.0.1"},
 		ClientConfig: snmp.ClientConfig{
@@ -261,31 +301,31 @@ func TestGetSNMPConnection_v2(t *testing.T) {
 			Community: "foo",
 		},
 	}
-	err := s.init()
+	err := s.Init()
 	require.NoError(t, err)
 
 	gsc, err := s.getConnection(0)
 	require.NoError(t, err)
 	gs := gsc.(snmp.GosnmpWrapper)
-	assert.Equal(t, "1.2.3.4", gs.Target)
-	assert.EqualValues(t, 567, gs.Port)
-	assert.Equal(t, gosnmp.Version2c, gs.Version)
-	assert.Equal(t, "foo", gs.Community)
-	assert.Equal(t, "udp", gs.Transport)
+	require.Equal(t, "1.2.3.4", gs.Target)
+	require.EqualValues(t, 567, gs.Port)
+	require.Equal(t, gosnmp.Version2c, gs.Version)
+	require.Equal(t, "foo", gs.Community)
+	require.Equal(t, "udp", gs.Transport)
 
 	gsc, err = s.getConnection(1)
 	require.NoError(t, err)
 	gs = gsc.(snmp.GosnmpWrapper)
-	assert.Equal(t, "1.2.3.4", gs.Target)
-	assert.EqualValues(t, 161, gs.Port)
-	assert.Equal(t, "udp", gs.Transport)
+	require.Equal(t, "1.2.3.4", gs.Target)
+	require.EqualValues(t, 161, gs.Port)
+	require.Equal(t, "udp", gs.Transport)
 
 	gsc, err = s.getConnection(2)
 	require.NoError(t, err)
 	gs = gsc.(snmp.GosnmpWrapper)
-	assert.Equal(t, "127.0.0.1", gs.Target)
-	assert.EqualValues(t, 161, gs.Port)
-	assert.Equal(t, "udp", gs.Transport)
+	require.Equal(t, "127.0.0.1", gs.Target)
+	require.EqualValues(t, 161, gs.Port)
+	require.Equal(t, "udp", gs.Transport)
 }
 
 func TestGetSNMPConnectionTCP(t *testing.T) {
@@ -297,16 +337,16 @@ func TestGetSNMPConnectionTCP(t *testing.T) {
 	s := &Snmp{
 		Agents: []string{"tcp://127.0.0.1:56789"},
 	}
-	err := s.init()
+	err := s.Init()
 	require.NoError(t, err)
 
 	wg.Add(1)
 	gsc, err := s.getConnection(0)
 	require.NoError(t, err)
 	gs := gsc.(snmp.GosnmpWrapper)
-	assert.Equal(t, "127.0.0.1", gs.Target)
-	assert.EqualValues(t, 56789, gs.Port)
-	assert.Equal(t, "tcp", gs.Transport)
+	require.Equal(t, "127.0.0.1", gs.Target)
+	require.EqualValues(t, 56789, gs.Port)
+	require.Equal(t, "tcp", gs.Transport)
 	wg.Wait()
 }
 
@@ -321,6 +361,10 @@ func stubTCPServer(wg *sync.WaitGroup) {
 }
 
 func TestGetSNMPConnection_v3(t *testing.T) {
+	if testing.Short() {
+		t.Skip("Skipping integration test in short mode")
+	}
+
 	s := &Snmp{
 		Agents: []string{"1.2.3.4"},
 		ClientConfig: snmp.ClientConfig{
@@ -338,29 +382,33 @@ func TestGetSNMPConnection_v3(t *testing.T) {
 			EngineTime:     2,
 		},
 	}
-	err := s.init()
+	err := s.Init()
 	require.NoError(t, err)
 
 	gsc, err := s.getConnection(0)
 	require.NoError(t, err)
 	gs := gsc.(snmp.GosnmpWrapper)
-	assert.Equal(t, gs.Version, gosnmp.Version3)
+	require.Equal(t, gs.Version, gosnmp.Version3)
 	sp := gs.SecurityParameters.(*gosnmp.UsmSecurityParameters)
-	assert.Equal(t, "1.2.3.4", gsc.Host())
-	assert.EqualValues(t, 20, gs.MaxRepetitions)
-	assert.Equal(t, "mycontext", gs.ContextName)
-	assert.Equal(t, gosnmp.AuthPriv, gs.MsgFlags&gosnmp.AuthPriv)
-	assert.Equal(t, "myuser", sp.UserName)
-	assert.Equal(t, gosnmp.MD5, sp.AuthenticationProtocol)
-	assert.Equal(t, "password123", sp.AuthenticationPassphrase)
-	assert.Equal(t, gosnmp.DES, sp.PrivacyProtocol)
-	assert.Equal(t, "321drowssap", sp.PrivacyPassphrase)
-	assert.Equal(t, "myengineid", sp.AuthoritativeEngineID)
-	assert.EqualValues(t, 1, sp.AuthoritativeEngineBoots)
-	assert.EqualValues(t, 2, sp.AuthoritativeEngineTime)
+	require.Equal(t, "1.2.3.4", gsc.Host())
+	require.EqualValues(t, 20, gs.MaxRepetitions)
+	require.Equal(t, "mycontext", gs.ContextName)
+	require.Equal(t, gosnmp.AuthPriv, gs.MsgFlags&gosnmp.AuthPriv)
+	require.Equal(t, "myuser", sp.UserName)
+	require.Equal(t, gosnmp.MD5, sp.AuthenticationProtocol)
+	require.Equal(t, "password123", sp.AuthenticationPassphrase)
+	require.Equal(t, gosnmp.DES, sp.PrivacyProtocol)
+	require.Equal(t, "321drowssap", sp.PrivacyPassphrase)
+	require.Equal(t, "myengineid", sp.AuthoritativeEngineID)
+	require.EqualValues(t, 1, sp.AuthoritativeEngineBoots)
+	require.EqualValues(t, 2, sp.AuthoritativeEngineTime)
 }
 
 func TestGetSNMPConnection_v3_blumenthal(t *testing.T) {
+	if testing.Short() {
+		t.Skip("Skipping integration test in short mode")
+	}
+
 	testCases := []struct {
 		Name      string
 		Algorithm gosnmp.SnmpV3PrivProtocol
@@ -455,35 +503,39 @@ func TestGetSNMPConnection_v3_blumenthal(t *testing.T) {
 	for _, tc := range testCases {
 		t.Run(tc.Name, func(t *testing.T) {
 			s := tc.Config
-			err := s.init()
+			err := s.Init()
 			require.NoError(t, err)
 
 			gsc, err := s.getConnection(0)
 			require.NoError(t, err)
 			gs := gsc.(snmp.GosnmpWrapper)
-			assert.Equal(t, gs.Version, gosnmp.Version3)
+			require.Equal(t, gs.Version, gosnmp.Version3)
 			sp := gs.SecurityParameters.(*gosnmp.UsmSecurityParameters)
-			assert.Equal(t, "1.2.3.4", gsc.Host())
-			assert.EqualValues(t, 20, gs.MaxRepetitions)
-			assert.Equal(t, "mycontext", gs.ContextName)
-			assert.Equal(t, gosnmp.AuthPriv, gs.MsgFlags&gosnmp.AuthPriv)
-			assert.Equal(t, "myuser", sp.UserName)
-			assert.Equal(t, gosnmp.MD5, sp.AuthenticationProtocol)
-			assert.Equal(t, "password123", sp.AuthenticationPassphrase)
-			assert.Equal(t, tc.Algorithm, sp.PrivacyProtocol)
-			assert.Equal(t, "password123", sp.PrivacyPassphrase)
-			assert.Equal(t, "myengineid", sp.AuthoritativeEngineID)
-			assert.EqualValues(t, 1, sp.AuthoritativeEngineBoots)
-			assert.EqualValues(t, 2, sp.AuthoritativeEngineTime)
+			require.Equal(t, "1.2.3.4", gsc.Host())
+			require.EqualValues(t, 20, gs.MaxRepetitions)
+			require.Equal(t, "mycontext", gs.ContextName)
+			require.Equal(t, gosnmp.AuthPriv, gs.MsgFlags&gosnmp.AuthPriv)
+			require.Equal(t, "myuser", sp.UserName)
+			require.Equal(t, gosnmp.MD5, sp.AuthenticationProtocol)
+			require.Equal(t, "password123", sp.AuthenticationPassphrase)
+			require.Equal(t, tc.Algorithm, sp.PrivacyProtocol)
+			require.Equal(t, "password123", sp.PrivacyPassphrase)
+			require.Equal(t, "myengineid", sp.AuthoritativeEngineID)
+			require.EqualValues(t, 1, sp.AuthoritativeEngineBoots)
+			require.EqualValues(t, 2, sp.AuthoritativeEngineTime)
 		})
 	}
 }
 
 func TestGetSNMPConnection_caching(t *testing.T) {
+	if testing.Short() {
+		t.Skip("Skipping integration test in short mode")
+	}
+
 	s := &Snmp{
 		Agents: []string{"1.2.3.4", "1.2.3.5", "1.2.3.5"},
 	}
-	err := s.init()
+	err := s.Init()
 	require.NoError(t, err)
 	gs1, err := s.getConnection(0)
 	require.NoError(t, err)
@@ -493,9 +545,9 @@ func TestGetSNMPConnection_caching(t *testing.T) {
 	require.NoError(t, err)
 	gs4, err := s.getConnection(2)
 	require.NoError(t, err)
-	assert.True(t, gs1 == gs2)
-	assert.False(t, gs2 == gs3)
-	assert.False(t, gs3 == gs4)
+	require.Equal(t, gs1, gs2)
+	require.NotEqual(t, gs2, gs3)
+	require.NotEqual(t, gs3, gs4)
 }
 
 func TestGosnmpWrapper_walk_retry(t *testing.T) {
@@ -545,11 +597,11 @@ func TestGosnmpWrapper_walk_retry(t *testing.T) {
 		GoSNMP: gs,
 	}
 	err = gsw.Walk(".1.0.0", func(_ gosnmp.SnmpPDU) error { return nil })
-	assert.NoError(t, srvr.Close())
+	require.NoError(t, srvr.Close())
 	wg.Wait()
-	assert.Error(t, err)
-	assert.False(t, gs.Conn == conn)
-	assert.Equal(t, (gs.Retries+1)*2, reqCount)
+	require.Error(t, err)
+	require.NotEqual(t, gs.Conn, conn)
+	require.Equal(t, (gs.Retries+1)*2, reqCount)
 }
 
 func TestGosnmpWrapper_get_retry(t *testing.T) {
@@ -600,12 +652,12 @@ func TestGosnmpWrapper_get_retry(t *testing.T) {
 	_, err = gsw.Get([]string{".1.0.0"})
 	require.NoError(t, srvr.Close())
 	wg.Wait()
-	assert.Error(t, err)
-	assert.False(t, gs.Conn == conn)
-	assert.Equal(t, (gs.Retries+1)*2, reqCount)
+	require.Error(t, err)
+	require.NotEqual(t, gs.Conn, conn)
+	require.Equal(t, (gs.Retries+1)*2, reqCount)
 }
 
-func TestTableBuild_walk(t *testing.T) {
+func TestTableBuild_walk_noTranslate(t *testing.T) {
 	tbl := Table{
 		Name:       "mytable",
 		IndexAsTag: true,
@@ -634,23 +686,12 @@ func TestTableBuild_walk(t *testing.T) {
 				Oid:            ".1.0.0.2.1.5",
 				OidIndexLength: 1,
 			},
-			{
-				Name:      "myfield6",
-				Oid:       ".1.0.0.0.1.6",
-				Translate: true,
-			},
-			{
-				Name:      "myfield7",
-				Oid:       ".1.0.0.0.1.6",
-				Translate: false,
-			},
 		},
 	}
 
 	tb, err := tbl.Build(tsc, true)
 	require.NoError(t, err)
-
-	assert.Equal(t, tb.Name, "mytable")
+	require.Equal(t, tb.Name, "mytable")
 	rtr1 := RTableRow{
 		Tags: map[string]string{
 			"myfield1": "foo",
@@ -661,8 +702,6 @@ func TestTableBuild_walk(t *testing.T) {
 			"myfield3": float64(0.123),
 			"myfield4": 11,
 			"myfield5": 11,
-			"myfield6": "testTableEntry.7",
-			"myfield7": ".1.0.0.0.1.7",
 		},
 	}
 	rtr2 := RTableRow{
@@ -694,11 +733,85 @@ func TestTableBuild_walk(t *testing.T) {
 			"myfield3": float64(9.999),
 		},
 	}
-	assert.Len(t, tb.Rows, 4)
-	assert.Contains(t, tb.Rows, rtr1)
-	assert.Contains(t, tb.Rows, rtr2)
-	assert.Contains(t, tb.Rows, rtr3)
-	assert.Contains(t, tb.Rows, rtr4)
+	require.Len(t, tb.Rows, 4)
+	require.Contains(t, tb.Rows, rtr1)
+	require.Contains(t, tb.Rows, rtr2)
+	require.Contains(t, tb.Rows, rtr3)
+	require.Contains(t, tb.Rows, rtr4)
+}
+
+func TestTableBuild_walk_Translate(t *testing.T) {
+	testDataPath, err := filepath.Abs("./testdata")
+	require.NoError(t, err)
+	s := &Snmp{
+		ClientConfig: snmp.ClientConfig{
+			Path: []string{testDataPath},
+		},
+	}
+	err = s.Init()
+	require.NoError(t, err)
+
+	tbl := Table{
+		Name:       "atTable",
+		IndexAsTag: true,
+		Fields: []Field{
+			{
+				Name:  "ifIndex",
+				Oid:   "1.3.6.1.2.1.3.1.1.1",
+				IsTag: true,
+			},
+			{
+				Name:      "atPhysAddress",
+				Oid:       "1.3.6.1.2.1.3.1.1.2",
+				Translate: false,
+			},
+			{
+				Name:      "atNetAddress",
+				Oid:       "1.3.6.1.2.1.3.1.1.3",
+				Translate: true,
+			},
+		},
+	}
+
+	err = tbl.Init()
+	require.NoError(t, err)
+	tb, err := tbl.Build(tsc, true)
+	require.NoError(t, err)
+
+	require.Equal(t, tb.Name, "atTable")
+
+	rtr1 := RTableRow{
+		Tags: map[string]string{
+			"ifIndex": "foo",
+			"index":   "0",
+		},
+		Fields: map[string]interface{}{
+			"atPhysAddress": 1,
+			"atNetAddress":  "atNetAddress",
+		},
+	}
+	rtr2 := RTableRow{
+		Tags: map[string]string{
+			"ifIndex": "bar",
+			"index":   "1",
+		},
+		Fields: map[string]interface{}{
+			"atPhysAddress": 2,
+		},
+	}
+	rtr3 := RTableRow{
+		Tags: map[string]string{
+			"index": "2",
+		},
+		Fields: map[string]interface{}{
+			"atPhysAddress": 0,
+		},
+	}
+
+	require.Len(t, tb.Rows, 3)
+	require.Contains(t, tb.Rows, rtr1)
+	require.Contains(t, tb.Rows, rtr2)
+	require.Contains(t, tb.Rows, rtr3)
 }
 
 func TestTableBuild_noWalk(t *testing.T) {
@@ -737,8 +850,8 @@ func TestTableBuild_noWalk(t *testing.T) {
 		Tags:   map[string]string{"myfield1": "baz", "myfield3": "234"},
 		Fields: map[string]interface{}{"myfield2": 234},
 	}
-	assert.Len(t, tb.Rows, 1)
-	assert.Contains(t, tb.Rows, rtr)
+	require.Len(t, tb.Rows, 1)
+	require.Contains(t, tb.Rows, rtr)
 }
 
 func TestGather(t *testing.T) {
@@ -776,7 +889,6 @@ func TestGather(t *testing.T) {
 		connectionCache: []snmpConnection{
 			tsc,
 		},
-		initialized: true,
 	}
 	acc := &testutil.Accumulator{}
 
@@ -787,21 +899,21 @@ func TestGather(t *testing.T) {
 	require.Len(t, acc.Metrics, 2)
 
 	m := acc.Metrics[0]
-	assert.Equal(t, "mytable", m.Measurement)
-	assert.Equal(t, "tsc", m.Tags[s.AgentHostTag])
-	assert.Equal(t, "baz", m.Tags["myfield1"])
-	assert.Len(t, m.Fields, 2)
-	assert.Equal(t, 234, m.Fields["myfield2"])
-	assert.Equal(t, "baz", m.Fields["myfield3"])
-	assert.True(t, !tstart.After(m.Time))
-	assert.True(t, !tstop.Before(m.Time))
+	require.Equal(t, "mytable", m.Measurement)
+	require.Equal(t, "tsc", m.Tags[s.AgentHostTag])
+	require.Equal(t, "baz", m.Tags["myfield1"])
+	require.Len(t, m.Fields, 2)
+	require.Equal(t, 234, m.Fields["myfield2"])
+	require.Equal(t, "baz", m.Fields["myfield3"])
+	require.False(t, tstart.After(m.Time))
+	require.False(t, tstop.Before(m.Time))
 
 	m2 := acc.Metrics[1]
-	assert.Equal(t, "myOtherTable", m2.Measurement)
-	assert.Equal(t, "tsc", m2.Tags[s.AgentHostTag])
-	assert.Equal(t, "baz", m2.Tags["myfield1"])
-	assert.Len(t, m2.Fields, 1)
-	assert.Equal(t, 123456, m2.Fields["myOtherField"])
+	require.Equal(t, "myOtherTable", m2.Measurement)
+	require.Equal(t, "tsc", m2.Tags[s.AgentHostTag])
+	require.Equal(t, "baz", m2.Tags["myfield1"])
+	require.Len(t, m2.Fields, 1)
+	require.Equal(t, 123456, m2.Fields["myOtherField"])
 }
 
 func TestGather_host(t *testing.T) {
@@ -823,7 +935,6 @@ func TestGather_host(t *testing.T) {
 		connectionCache: []snmpConnection{
 			tsc,
 		},
-		initialized: true,
 	}
 
 	acc := &testutil.Accumulator{}
@@ -832,7 +943,7 @@ func TestGather_host(t *testing.T) {
 
 	require.Len(t, acc.Metrics, 1)
 	m := acc.Metrics[0]
-	assert.Equal(t, "baz", m.Tags["host"])
+	require.Equal(t, "baz", m.Tags["host"])
 }
 
 func TestFieldConvert(t *testing.T) {
@@ -841,11 +952,12 @@ func TestFieldConvert(t *testing.T) {
 		conv     string
 		expected interface{}
 	}{
-		{[]byte("foo"), "", string("foo")},
+		{[]byte("foo"), "", "foo"},
 		{"0.123", "float", float64(0.123)},
 		{[]byte("0.123"), "float", float64(0.123)},
 		{float32(0.123), "float", float64(float32(0.123))},
 		{float64(0.123), "float", float64(0.123)},
+		{float64(0.123123123123), "float", float64(0.123123123123)},
 		{123, "float", float64(123)},
 		{123, "float(0)", float64(123)},
 		{123, "float(4)", float64(0.0123)},
@@ -864,7 +976,7 @@ func TestFieldConvert(t *testing.T) {
 		{[]byte("123123123123"), "int", int64(123123123123)},
 		{float32(12.3), "int", int64(12)},
 		{float64(12.3), "int", int64(12)},
-		{int(123), "int", int64(123)},
+		{123, "int", int64(123)},
 		{int8(123), "int", int64(123)},
 		{int16(123), "int", int64(123)},
 		{int32(123), "int", int64(123)},
@@ -889,25 +1001,23 @@ func TestFieldConvert(t *testing.T) {
 
 	for _, tc := range testTable {
 		act, err := fieldConvert(tc.conv, tc.input)
-		if !assert.NoError(t, err, "input=%T(%v) conv=%s expected=%T(%v)", tc.input, tc.input, tc.conv, tc.expected, tc.expected) {
-			continue
-		}
-		assert.EqualValues(t, tc.expected, act, "input=%T(%v) conv=%s expected=%T(%v)", tc.input, tc.input, tc.conv, tc.expected, tc.expected)
+		require.NoError(t, err, "input=%T(%v) conv=%s expected=%T(%v)", tc.input, tc.input, tc.conv, tc.expected, tc.expected)
+		require.EqualValues(t, tc.expected, act, "input=%T(%v) conv=%s expected=%T(%v)", tc.input, tc.input, tc.conv, tc.expected, tc.expected)
 	}
 }
 
 func TestSnmpTranslateCache_miss(t *testing.T) {
 	snmpTranslateCaches = nil
 	oid := "IF-MIB::ifPhysAddress.1"
-	mibName, oidNum, oidText, conversion, err := SnmpTranslate(oid)
-	assert.Len(t, snmpTranslateCaches, 1)
+	mibName, oidNum, oidText, conversion, _, err := SnmpTranslate(oid)
+	require.Len(t, snmpTranslateCaches, 1)
 	stc := snmpTranslateCaches[oid]
 	require.NotNil(t, stc)
-	assert.Equal(t, mibName, stc.mibName)
-	assert.Equal(t, oidNum, stc.oidNum)
-	assert.Equal(t, oidText, stc.oidText)
-	assert.Equal(t, conversion, stc.conversion)
-	assert.Equal(t, err, stc.err)
+	require.Equal(t, mibName, stc.mibName)
+	require.Equal(t, oidNum, stc.oidNum)
+	require.Equal(t, oidText, stc.oidText)
+	require.Equal(t, conversion, stc.conversion)
+	require.Equal(t, err, stc.err)
 }
 
 func TestSnmpTranslateCache_hit(t *testing.T) {
@@ -920,12 +1030,12 @@ func TestSnmpTranslateCache_hit(t *testing.T) {
 			err:        fmt.Errorf("e"),
 		},
 	}
-	mibName, oidNum, oidText, conversion, err := SnmpTranslate("foo")
-	assert.Equal(t, "a", mibName)
-	assert.Equal(t, "b", oidNum)
-	assert.Equal(t, "c", oidText)
-	assert.Equal(t, "d", conversion)
-	assert.Equal(t, fmt.Errorf("e"), err)
+	mibName, oidNum, oidText, conversion, _, err := SnmpTranslate("foo")
+	require.Equal(t, "a", mibName)
+	require.Equal(t, "b", oidNum)
+	require.Equal(t, "c", oidText)
+	require.Equal(t, "d", conversion)
+	require.Equal(t, fmt.Errorf("e"), err)
 	snmpTranslateCaches = nil
 }
 
@@ -933,14 +1043,14 @@ func TestSnmpTableCache_miss(t *testing.T) {
 	snmpTableCaches = nil
 	oid := ".1.0.0.0"
 	mibName, oidNum, oidText, fields, err := snmpTable(oid)
-	assert.Len(t, snmpTableCaches, 1)
+	require.Len(t, snmpTableCaches, 1)
 	stc := snmpTableCaches[oid]
 	require.NotNil(t, stc)
-	assert.Equal(t, mibName, stc.mibName)
-	assert.Equal(t, oidNum, stc.oidNum)
-	assert.Equal(t, oidText, stc.oidText)
-	assert.Equal(t, fields, stc.fields)
-	assert.Equal(t, err, stc.err)
+	require.Equal(t, mibName, stc.mibName)
+	require.Equal(t, oidNum, stc.oidNum)
+	require.Equal(t, oidText, stc.oidText)
+	require.Equal(t, fields, stc.fields)
+	require.Equal(t, err, stc.err)
 }
 
 func TestSnmpTableCache_hit(t *testing.T) {
@@ -954,9 +1064,275 @@ func TestSnmpTableCache_hit(t *testing.T) {
 		},
 	}
 	mibName, oidNum, oidText, fields, err := snmpTable("foo")
-	assert.Equal(t, "a", mibName)
-	assert.Equal(t, "b", oidNum)
-	assert.Equal(t, "c", oidText)
-	assert.Equal(t, []Field{{Name: "d"}}, fields)
-	assert.Equal(t, fmt.Errorf("e"), err)
+	require.Equal(t, "a", mibName)
+	require.Equal(t, "b", oidNum)
+	require.Equal(t, "c", oidText)
+	require.Equal(t, []Field{{Name: "d"}}, fields)
+	require.Equal(t, fmt.Errorf("e"), err)
+}
+
+func TestTableJoin_walk(t *testing.T) {
+	tbl := Table{
+		Name:       "mytable",
+		IndexAsTag: true,
+		Fields: []Field{
+			{
+				Name:  "myfield1",
+				Oid:   ".1.0.0.3.1.1",
+				IsTag: true,
+			},
+			{
+				Name: "myfield2",
+				Oid:  ".1.0.0.3.1.2",
+			},
+			{
+				Name:                "myfield3",
+				Oid:                 ".1.0.0.3.1.3",
+				SecondaryIndexTable: true,
+			},
+			{
+				Name:              "myfield4",
+				Oid:               ".1.0.0.0.1.1",
+				SecondaryIndexUse: true,
+				IsTag:             true,
+			},
+			{
+				Name:              "myfield5",
+				Oid:               ".1.0.0.0.1.2",
+				SecondaryIndexUse: true,
+			},
+		},
+	}
+
+	tb, err := tbl.Build(tsc, true)
+	require.NoError(t, err)
+
+	require.Equal(t, tb.Name, "mytable")
+	rtr1 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance",
+			"myfield4": "bar",
+			"index":    "10",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 10,
+			"myfield3": 1,
+			"myfield5": 2,
+		},
+	}
+	rtr2 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance2",
+			"index":    "11",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 20,
+			"myfield3": 2,
+			"myfield5": 0,
+		},
+	}
+	rtr3 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance3",
+			"index":    "12",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 20,
+			"myfield3": 3,
+		},
+	}
+	require.Len(t, tb.Rows, 3)
+	require.Contains(t, tb.Rows, rtr1)
+	require.Contains(t, tb.Rows, rtr2)
+	require.Contains(t, tb.Rows, rtr3)
+}
+
+func TestTableOuterJoin_walk(t *testing.T) {
+	tbl := Table{
+		Name:       "mytable",
+		IndexAsTag: true,
+		Fields: []Field{
+			{
+				Name:  "myfield1",
+				Oid:   ".1.0.0.3.1.1",
+				IsTag: true,
+			},
+			{
+				Name: "myfield2",
+				Oid:  ".1.0.0.3.1.2",
+			},
+			{
+				Name:                "myfield3",
+				Oid:                 ".1.0.0.3.1.3",
+				SecondaryIndexTable: true,
+				SecondaryOuterJoin:  true,
+			},
+			{
+				Name:              "myfield4",
+				Oid:               ".1.0.0.0.1.1",
+				SecondaryIndexUse: true,
+				IsTag:             true,
+			},
+			{
+				Name:              "myfield5",
+				Oid:               ".1.0.0.0.1.2",
+				SecondaryIndexUse: true,
+			},
+		},
+	}
+
+	tb, err := tbl.Build(tsc, true)
+	require.NoError(t, err)
+
+	require.Equal(t, tb.Name, "mytable")
+	rtr1 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance",
+			"myfield4": "bar",
+			"index":    "10",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 10,
+			"myfield3": 1,
+			"myfield5": 2,
+		},
+	}
+	rtr2 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance2",
+			"index":    "11",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 20,
+			"myfield3": 2,
+			"myfield5": 0,
+		},
+	}
+	rtr3 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance3",
+			"index":    "12",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 20,
+			"myfield3": 3,
+		},
+	}
+	rtr4 := RTableRow{
+		Tags: map[string]string{
+			"index":    "Secondary.0",
+			"myfield4": "foo",
+		},
+		Fields: map[string]interface{}{
+			"myfield5": 1,
+		},
+	}
+	require.Len(t, tb.Rows, 4)
+	require.Contains(t, tb.Rows, rtr1)
+	require.Contains(t, tb.Rows, rtr2)
+	require.Contains(t, tb.Rows, rtr3)
+	require.Contains(t, tb.Rows, rtr4)
+}
+
+func TestTableJoinNoIndexAsTag_walk(t *testing.T) {
+	tbl := Table{
+		Name:       "mytable",
+		IndexAsTag: false,
+		Fields: []Field{
+			{
+				Name:  "myfield1",
+				Oid:   ".1.0.0.3.1.1",
+				IsTag: true,
+			},
+			{
+				Name: "myfield2",
+				Oid:  ".1.0.0.3.1.2",
+			},
+			{
+				Name:                "myfield3",
+				Oid:                 ".1.0.0.3.1.3",
+				SecondaryIndexTable: true,
+			},
+			{
+				Name:              "myfield4",
+				Oid:               ".1.0.0.0.1.1",
+				SecondaryIndexUse: true,
+				IsTag:             true,
+			},
+			{
+				Name:              "myfield5",
+				Oid:               ".1.0.0.0.1.2",
+				SecondaryIndexUse: true,
+			},
+		},
+	}
+
+	tb, err := tbl.Build(tsc, true)
+	require.NoError(t, err)
+
+	require.Equal(t, tb.Name, "mytable")
+	rtr1 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance",
+			"myfield4": "bar",
+			//"index":    "10",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 10,
+			"myfield3": 1,
+			"myfield5": 2,
+		},
+	}
+	rtr2 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance2",
+			//"index":    "11",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 20,
+			"myfield3": 2,
+			"myfield5": 0,
+		},
+	}
+	rtr3 := RTableRow{
+		Tags: map[string]string{
+			"myfield1": "instance3",
+			//"index":    "12",
+		},
+		Fields: map[string]interface{}{
+			"myfield2": 20,
+			"myfield3": 3,
+		},
+	}
+	require.Len(t, tb.Rows, 3)
+	require.Contains(t, tb.Rows, rtr1)
+	require.Contains(t, tb.Rows, rtr2)
+	require.Contains(t, tb.Rows, rtr3)
+}
+
+func BenchmarkMibLoading(b *testing.B) {
+	log := testutil.Logger{}
+	path := []string{"testdata"}
+	for i := 0; i < b.N; i++ {
+		err := snmp.LoadMibsFromPath(path, log, &snmp.GosmiMibLoader{})
+		require.NoError(b, err)
+	}
+}
+
+func TestCanNotParse(t *testing.T) {
+	s := &Snmp{
+		Fields: []Field{
+			{Oid: "RFC1213-MIB::"},
+		},
+	}
+
+	err := s.Init()
+	require.Error(t, err)
+}
+
+func TestMissingMibPath(t *testing.T) {
+	log := testutil.Logger{}
+	path := []string{"non-existing-directory"}
+	err := snmp.LoadMibsFromPath(path, log, &snmp.GosmiMibLoader{})
+	require.NoError(t, err)
 }
